@@ -1,6 +1,6 @@
 package XML::ExtOn;
 
-#$Id: ExtOn.pm 605 2009-09-08 09:13:46Z zag $
+#$Id: ExtOn.pm 694 2010-01-18 17:47:09Z zag $
 
 =pod
 
@@ -193,7 +193,7 @@ require Exporter;
 sub _get_end_handler {
     my $flt     = shift;
     my $handler = $flt->get_handler();
-#    warn "$flt -> $handler";
+
     return $handler if UNIVERSAL::isa( $handler, 'XML::SAX::Writer::XML' );
     return $flt unless UNIVERSAL::isa( $handler, 'XML::SAX::Base' );
     return &_get_end_handler($handler);
@@ -238,7 +238,8 @@ sub create_pipe {
                 $end_handler->set_handler($out_handler);
                 $out_handler = $f;
             }
-        } else {
+        }
+        else {
             die "$f not SAX Drv";
         }
     }
@@ -247,9 +248,11 @@ sub create_pipe {
 
 use base 'XML::SAX::Base';
 use vars qw( $AUTOLOAD);
-$XML::ExtOn::VERSION = '0.11';
+$XML::ExtOn::VERSION = '0.12';
 ### install get/set accessors for this object.
-for my $key (qw/ context _objects_stack _cdata_mode _cdata_characters/) {
+for my $key (
+    qw/ context _objects_stack _cdata_mode _cdata_characters _root_stack /)
+{
     no strict 'refs';
     *{ __PACKAGE__ . "::$key" } = sub {
         my $self = shift;
@@ -266,6 +269,7 @@ sub new {
     my $class = shift;
     my $self = &XML::SAX::Base::new( $class, @_, );
     $self->_objects_stack( [] );
+    $self->_root_stack( [] );    #init incoming stack of start end
     $self->_cdata_mode(0);
     my $buf;
     $self->_cdata_characters( \$buf );    #setup cdata buffer
@@ -339,15 +343,9 @@ sub start_prefix_mapping {
     my $self = shift;
 
     #declare namespace for current context
-    #    my $context = $self->context;
-    #    if ( my $current = $self->current_element ) {
-    #        $context = $current->ns;
-    #    }
     my %map = ();
     foreach my $ref (@_) {
         my ( $prefix, $ns_uri ) = @{$ref}{qw/Prefix NamespaceURI/};
-
-        #        $context->declare_prefix( $prefix, $ns_uri );
         $map{$prefix} = $ns_uri;
     }
     $self->on_start_prefix_mapping(%map);
@@ -381,69 +379,163 @@ sub on_start_element {
     return [@_];
 }
 
-sub start_element {
+sub __expand_on_start {
     my $self = shift;
-    my $data = shift;
-
-    #check current element for skip_content
-    if ( my $current_element = $self->current_element ) {
-        my $skip_content = $current_element->is_skip_content;
-        if ($skip_content) {
-            $current_element->is_skip_content( ++$skip_content );
-            return;
-        }
-    }
-
-    my $current_obj;
-    unless ( UNIVERSAL::isa( $data, 'XML::ExtOn::Element' ) ) {
-        $current_obj = $self->__mk_element_from_sax2($data);
-    }
-    else {
-        $current_obj = $data;
-
-        #set new context
-        my $new_context;
-        if ( my $current_element = $self->current_element ) {
-            $new_context = $current_element->ns->sub_context();
-        }
-        $new_context ||= $self->context->sub_context();
-        $current_obj->_context($new_context);
-    }
-    my $res = $self->on_start_element($current_obj);
+    my $obj  = shift || return [];
+    my $res  = $self->on_start_element($obj);
     my @stack =
         $res
       ? ref($res) eq 'ARRAY'
           ? @{$res}
           : ($res)
       : ();
-    push @stack, $current_obj;
+
+    #add self object
+    push @stack, $obj;
+
+    #expand wrap_around and insert_to
+    # also remove dups for $obj
     my %uniq = ();
+    my @res  = ();
+    foreach my $o (@stack) {
 
-    #process answer
-    foreach my $elem (@stack) {
+        # also remove dups for $obj
+        next if $uniq{$o}++;
+        unless ( UNIVERSAL::isa( $o, 'XML::ExtOn::Element' ) ) {
 
-        #clean dups
-        next if $uniq{$elem}++;
-        unless ( $elem eq $current_obj ) {
+            #don'n touch any events
+            push @res, $o;
+        }
+        else {
 
-            #               warn "++".$elem->local_name;
+            #convert any object to events (exept $obj)
+            unless ( $o eq $obj ) {
+                push @res, $self->mk_start_element($o),
+                  $self->mk_process_stack($o), $self->mk_end_element($o);
+            }
+            else {
+
+                #expand $insert_to
+                my $insert_to = $o->_wrap_begin || [];
+                if ( scalar @{$insert_to} ) {
+                    for ( @{$insert_to} ) {
+                        push @res, $self->mk_start_element($_);
+                    }
+                }
+
+                #                $o->_wrap_begin([]);
+                #insert result event to write tag
+                push @res, $self->_mk_event_start_element($o);
+
+                #process elemet's stack (add_content)
+                push @res, $self->mk_process_stack($o);
+
+                #ad  wrap_around started
+                my $waround = $o->_wrap_around_start || [];
+                if ( scalar @{$waround} ) {
+                    for ( @{$waround} ) {
+                        push @res, $self->mk_start_element($_);
+                    }
+                }
+
+                #                $o->_wrap_around_start([]);
+            }
+        }
+    }
+
+    #now expand
+    return \@res;
+}
+
+sub start_element {
+    my $self        = shift;
+    my $current_obj = shift;
+
+    die "empty" . Dumper( [ map { [ caller($_) ] } ( 0 .. 4 ) ] )
+      unless defined $current_obj;
+
+    unless ( UNIVERSAL::isa( $current_obj, 'XML::ExtOn::Element' ) ) {
+        my $context;
+        if ( my $current_root_element = $self->current_root_element ) {
+            $context = $current_root_element->ns->sub_context();
+        }
+        $current_obj =
+          $self->__mk_element_from_sax2( $current_obj, context => $context );
+    }
+    else {
+
+        #set new context
+        my $new_context;
+        if ( my $current_root_element = $self->current_root_element ) {
+            $new_context = $current_root_element->ns->sub_context();
+        }
+        $new_context ||= $self->context->sub_context();
+        $current_obj->_context($new_context);
+    }
+
+    my $current_root_element = $self->current_root_element;
+
+    #push to stack of incoming objects
+    push @{ $self->_root_stack() }, $current_obj;
+
+    #=comment check skip
+    #check current root element for skip_content
+    if ($current_root_element) {
+        my $skip_content = $current_root_element->is_skip_content;
+        if ($skip_content) {
+            $current_root_element->is_skip_content( ++$skip_content );
+            return;
+        }
+    }
+
+    #=cut
+    #warn ref($self).":START for " . $current_obj->local_name;
+    return $self->__start_element($current_obj);
+}
+
+sub __start_element {
+    my $self        = shift;
+    my $current_obj = shift;
+
+    #check current element for skip_content
+    if ( my $current_element = $self->current_element ) {
+        my $skip_content = $current_element->is_skip_content;
+        if ( $skip_content > 1 ) {
+            $current_element->is_skip_content( --$skip_content );
+            return;
+        }
+    }
+
+    #call __start_element
+    my $res = $self->__expand_on_start($current_obj);
+    $current_obj->{_expanded_on_start} = scalar(@$res);
+
+#    warn ref($self) . "start_exp: " . $current_obj->local_name . ": " . Dumper(
+#        [
+#            map {
+#                ref($_) eq 'HASH'
+#                  ? $_->{type} . ":" . $_->{data}->local_name
+#                  : $_->local_name
+#              } @$res
+#        ]
+#    );
+
+    #walk via array
+    foreach my $elem (@$res) {
+
+        unless ( UNIVERSAL::isa( $elem, 'XML::ExtOn::Element' ) ) {
+
+            #run event
+            #warn $elem->{type};
             $self->_process_comm($elem);
         }
         else {
-            if ( my $wrapper = $current_obj->_wrap_begin ) {
-                $current_obj->_wrap_begin(0);
-                $current_obj->_wrap_end($wrapper);
-                $self->start_element($wrapper);
-            }
 
             #register new namespaces
             my $changes    = $current_obj->ns->get_changes;
             my $parent_map = $current_obj->ns->parent->get_map;
 
-            #warn Dumper( { changes => $changes } );
             for ( keys %$changes ) {
-
-                #                $self->SUPER::end_prefix_mapping(
                 $self->end_prefix_mapping(
                     {
                         Prefix       => $_,
@@ -451,7 +543,6 @@ sub start_element {
                     }
                 ) if exists $parent_map->{$_};
 
-                #                $self->SUPER::start_prefix_mapping(
                 $self->start_prefix_mapping(
                     {
                         Prefix       => $_,
@@ -467,9 +558,17 @@ sub start_element {
 
             #skip deleted elements from xml stream
             unless ( $current_obj->is_delete_element ) {
+
+         #                warn "$self: process start ".$current_obj->local_name;
                 if ( UNIVERSAL::isa( $self->{Handler}, 'XML::ExtOn' ) ) {
                     my $cloned = $current_obj->__clone;
-                    $self->{Handler}->start_element($cloned);
+                    unless ( $self->{__make_self_events} ) {
+                        $self->{Handler}->start_element($cloned);
+                    }
+                    else {
+                        $self->{Handler}->__start_element($cloned);
+
+                    }
                 }
                 else {
                     my $res_data = $self->__exp_element_to_sax2($current_obj);
@@ -516,68 +615,163 @@ sub on_end_element {
     return [@_];
 }
 
-#process SUPER end_element
-sub __end_element {
+sub __expand_on_end {
     my $self = shift;
-    my @data = @_;
+    my $obj = shift || return [];
 
-    #    foreach my $elem {
     #
-    #    }
-}
-
-sub end_element {
-    my $self = shift;
-    my $data = shift;
-
-    #check current element for skip_content
-    if ( my $current_element = $self->current_element ) {
-        my $skip_content = $current_element->is_skip_content;
-        if ( $skip_content > 1 ) {
-            $current_element->is_skip_content( --$skip_content );
-            return;
-        }
-    }
-
-    #pop element from stack
-    my $current_obj = pop @{ $self->_objects_stack() };
-
-    #setup default ns
-    $data = $current_obj->to_sax2;
-    delete $data->{Attributes};
-    $data->{NamespaceURI} = $current_obj->default_uri;
-
-    my $res = $self->on_end_element($current_obj);
+    my $res = $self->on_end_element($obj);
     my @stack =
         $res
       ? ref($res) eq 'ARRAY'
           ? @{$res}
           : ($res)
       : ();
-    push @stack, $current_obj;
-    my %uniq             = ();
-    my $call_end_element = 0;
 
-    #process answer
-    foreach my $elem (@stack) {
+    #add self object
+    push @stack, $obj;
 
-        #clean dups
-        next if $uniq{$elem}++;
-        unless ( $elem eq $current_obj ) {
+    #expand wrap_around and insert_to
+    # also remove dups for $obj
+    my %uniq = ();
+    my @res  = ();
+    foreach my $o (@stack) {
+
+        # also remove dups for $obj
+        next if $uniq{$o}++;
+        unless ( UNIVERSAL::isa( $o, 'XML::ExtOn::Element' ) ) {
+
+            #don'n touch any events
+            push @res, $o;
+        }
+        else {
+
+            #convert any object to events (exept $obj)
+            unless ( $o eq $obj ) {
+                push @res, $self->mk_start_element($o),
+                  $self->mk_process_stack($o), $self->mk_end_element($o);
+            }
+            else {
+
+                #ad  wrap_around started
+                my $waround = $o->_wrap_around_end || [];
+                if ( scalar @{$waround} ) {
+                    for ( reverse @{$waround} ) {
+                        push @res, $self->mk_end_element($_);
+                    }
+                }
+
+                #                push @res, $o;    #add object
+                #process elemet's stack (add_content)
+                push @res, $self->mk_process_stack($o);
+
+                #expand $insert_to
+                push @res, $self->_mk_event_end_element($o);
+
+                my $insert_to = $o->_wrap_end || [];
+                if ( scalar @{$insert_to} ) {
+                    for ( reverse @{$insert_to} ) {
+                        push @res, $self->mk_end_element($_);
+                    }
+                }
+            }
+        }
+    }
+
+    #now expand
+    return \@res;
+}
+
+sub end_element {
+    my $self = shift;
+    my $data = shift;
+
+    #get current element
+    #pop from stack of incoming objects
+    $data = pop @{ $self->_root_stack() };
+    die " $self empty stack" . Dumper( [ map { [ caller($_) ] } ( 0 .. 4 ) ] )
+      unless defined $data;
+
+    #    warn "do __end; for "
+    #      . $data->local_name
+    #      . " {_expanded_on_start}"
+    #      . $data->{_expanded_on_start};
+
+    #check current element for skip_content
+    if ( my $current_root_element = $self->current_root_element ) {
+        my $skip_content = $current_root_element->is_skip_content;
+        if ( $skip_content > 1 ) {
+            $current_root_element->is_skip_content( --$skip_content );
+            return;
+        }
+    }
+
+#    warn ref($self).":END for " . $data->local_name;
+#    if ( my $started = $data->{_expanded_on_start} )  {
+#        for ( 1..$started-1 ) {
+#            $self->__end_element($data);
+#        }
+#    }
+    return $self->__end_element($data);
+}
+
+sub __end_element {
+    my $self = shift;
+
+    my $current_obj = shift;    #may be use for control stack
+                                #pop element from stack
+
+    #    my $current_obj1 = pop @{ $self->_objects_stack() };
+
+    my $res = $self->__expand_on_end($current_obj);
+
+#   warn ref($self)."end_exp: "
+#      . $current_obj->local_name . ": "
+#      . Dumper(
+#        [
+#            map { ref($_) eq 'HASH' ? $_->{type}.":".$_->{data}->local_name : $_->local_name }
+#              @$res
+#        ]
+#      );
+
+    foreach my $elem (@$res) {
+        unless ( UNIVERSAL::isa( $elem, 'XML::ExtOn::Element' ) ) {
+
+            #run event
             $self->_process_comm($elem);
         }
         else {
+            die "END!!";
+
+            #setup default ns
+            my $data = $current_obj->to_sax2;
+            delete $data->{Attributes};
+            $data->{NamespaceURI} = $current_obj->default_uri;
+
+            # if skip
+            #check current element for skip_content
+            if ( my $current_element = $self->current_element ) {
+                my $skip_content = $current_element->is_skip_content;
+                if ( $skip_content > 1 ) {
+                    $current_element->is_skip_content( --$skip_content );
+                    return;
+                }
+            }
+
             unless ( $current_obj->is_skip_content ) {
                 $self->_process_comm($_) for @{ $current_obj->_stack };
                 $current_obj->_stack( [] );
             }
-            $self->SUPER::end_element($data)
-              unless $current_obj->is_delete_element;
-            if ( my $wrapper = $current_obj->_wrap_end ) {
-                $current_obj->_wrap_end(0);
-                $self->end_element($wrapper);
 
-                #clear wrap_end attr
+            unless ( $current_obj->is_delete_element ) {
+
+           #                warn "$self: process end ".$current_obj->local_name;
+                unless ( $self->{__make_self_events} ) {
+                    $self->SUPER::end_element($data);
+                }
+                else {
+                    $self->{Handler}->__end_element($data);
+                }
             }
 
             my $changes    = $current_obj->ns->get_changes;
@@ -651,6 +845,7 @@ sub end_cdata {
     if ( my $elem = $self->current_element
         and defined( my $cdata_buf = ${ $self->_cdata_characters } ) )
     {
+
         if ( defined( my $data = $self->on_cdata( $elem, $cdata_buf ) ) ) {
             $self->SUPER::start_cdata;
             $self->SUPER::characters( { Data => $data } );
@@ -668,22 +863,24 @@ sub end_cdata {
 sub characters {
     my $self = shift;
     my ($data) = @_;
+#    warn "$self do chars" . $data->{Data};
 
-#skip childs elements characters ( > 1 ) and self text ( > 0)
-#    warn $self.Dumper([ map {[caller($_)]} (1..10)]) unless $self->current_element;
+    #skip childs elements characters ( > 1 ) and self text ( > 0)
     if ( $self->current_element ) {
         return if $self->current_element->is_skip_content;
     }
     else {
 
         #skip characters without element
-        return
-
-          #        #warn "characters without element"
+        return;
     }
 
     #for cdata section collect characters in buffer
     if ( $self->_cdata_mode ) {
+#        warn "$self do CDATA" . $data->{Data};
+#        warn  " $self CDTATA" . Dumper( [ map { [ caller($_) ] } ( 0 .. 10 ) ] );
+#      unless defined $data;
+
         ${ $self->_cdata_characters } .= $data->{Data};
         return;
     }
@@ -727,11 +924,12 @@ Return command  for include to stream.
 =cut
 
 sub mk_from_xml {
-    my $self          = shift;
-    my $string        = shift;
-    my $skip_tmp_root = XML::ExtOn::IncXML->new( Handler => $self );
+    my $self   = shift;
+    my $string = shift;
+    my $skip_tmp_root =
+      XML::ExtOn::IncXML->new( Handler => $self, __make_self_events => 1 );
     my $sax2_filter = XML::Filter::SAX1toSAX2->new( Handler => $skip_tmp_root );
-    my $parser      = XML::Parser::PerlSAX->new(
+    my $parser = XML::Parser::PerlSAX->new(
         {
             Handler => $sax2_filter,
             Source  => { String => "<tmp>$string</tmp>" }
@@ -776,6 +974,44 @@ sub mk_start_element {
     return { type => 'START_ELEMENT', data => $elem };
 }
 
+=head2 mk_event_element <element object>
+
+return command for expand stack for element
+
+=cut
+
+sub mk_process_stack {
+    my $self    = shift;
+    my $elem    = shift;
+    my @objects = @{ $elem->_stack };
+    $elem->_stack([]);
+    return { type => 'STACK', data => $elem, objects => \@objects };
+}
+
+=head2 _mk_event_start_element <element object>
+
+return start tag command. (internal)
+
+=cut
+
+sub _mk_event_start_element {
+    my $self = shift;
+    my $elem = shift;
+    return { type => 'EV_START_ELEMENT', data => $elem };
+}
+
+=head2 _mk_event_end_element <element object>
+
+return end tag command. (internal)
+
+=cut
+
+sub _mk_event_end_element {
+    my $self = shift;
+    my $elem = shift;
+    return { type => 'EV_END_ELEMENT', data => $elem };
+}
+
 =head2 mk_end_element <element object>
 
 return command for end element event
@@ -815,6 +1051,21 @@ sub current_element {
     return;
 }
 
+=head2 current_root_element 
+
+Return link to current root element  in incoming stack.
+Used in start_element and end_element methods
+
+=cut
+
+sub current_root_element {
+    my $self = shift;
+    if ( my $stack = $self->_root_stack() ) {
+        return $stack->[-1];
+    }
+    return;
+}
+
 # Private method for process commands
 
 sub _process_comm {
@@ -824,16 +1075,19 @@ sub _process_comm {
         $comm->parse;
     }
     elsif ( UNIVERSAL::isa( $comm, 'XML::ExtOn::Element' ) ) {
-        $self->start_element($comm);
+#        warn ref($self)."start ELEMENT " . $comm->local_name;
+        $self->__start_element($comm);
 
-        while ( my $obj = shift @{ $comm->_stack } ) {
-            $self->_process_comm($obj);
-        }
-        $self->end_element($comm)
+#        while ( my $obj = shift @{ $comm->_stack } ) {
+#            $self->_process_comm($obj);
+#        }
+        $self->__end_element($comm);
+#        warn ref($self)."end ELEMENT " . $comm->local_name;
           ;    # unless shift; #if exists extra param not end elem
     }
     elsif ( ref($comm) eq 'HASH' and exists $comm->{type} ) {
         if ( $comm->{type} eq 'CDATA' ) {
+            #warn "$self : DO CDATA!!!";
             $self->start_cdata;
             $self->characters( { Data => ${ $comm->{data} } } );
             $self->end_cdata;
@@ -843,21 +1097,135 @@ sub _process_comm {
         }
         elsif ( $comm->{type} eq 'START_ELEMENT' ) {
             my $current_obj = $comm->{data};
-            my $res_data    = $self->__exp_element_to_sax2($current_obj);
-#            $self->SUPER::start_element($res_data);
-
-            $self->start_element( $comm->{data} );
+            $self->__start_element( $comm->{data} );
         }
         elsif ( $comm->{type} eq 'END_ELEMENT' ) {
             my $current_obj = $comm->{data};
-            my $data        = $current_obj->to_sax2;
-            delete $data->{Attributes};
-            $data->{NamespaceURI} = $current_obj->default_uri;
-            $self->end_element( $comm->{data} )
-#             $self->SUPER::end_element($data)
-              unless $current_obj->is_delete_element;
+            $self->__end_element( $comm->{data} );
+        }
+        elsif ( $comm->{type} eq 'STACK' ) {
+            my $stack = $comm->{objects};
+            my $comm  = $comm->{data};
+#            warn "$self: ",
+#              $comm->local_name . " stack: " . scalar( @{$stack} ) . Dumper(
+#                [
+#                    map {
+#                        ref($_) eq 'HASH'
+#                          ? $_->{type} . ":" . '$_->{data}->local_name'
+#                          : $_->local_name
+#                      } @$stack
+#                ]
+#              );
+#            warn ref($self)."START PROCESS STACK ".$comm->local_name;
+            while ( my $obj = shift @{$stack} ) {
+
+#                warn "$self start STACK: ".$obj;
+                $self->_process_comm($obj);
+#                warn "$self end STACK: ".$obj;
+            }
+#            warn ref($self)."END PROCESS STACK ".$comm->local_name;
+
+        }
+        elsif ( $comm->{type} eq 'EV_START_ELEMENT' ) {
+            my $current_obj = $comm->{data};
+
+            #            warn "$self: ev_START".$current_obj->local_name;
+
+            #register new namespaces
+            my $changes    = $current_obj->ns->get_changes;
+            my $parent_map = $current_obj->ns->parent->get_map;
+
+            for ( keys %$changes ) {
+                $self->end_prefix_mapping(
+                    {
+                        Prefix       => $_,
+                        NamespaceURI => $parent_map->{$_},
+                    }
+                ) if exists $parent_map->{$_};
+
+                $self->start_prefix_mapping(
+                    {
+                        Prefix       => $_,
+                        NamespaceURI => $changes->{$_},
+                    }
+                );
+            }
+
+            #save element in stack
+            push @{ $self->_objects_stack() }, $current_obj;
+
+            #warn ref($self) . ": <" . $comm->{data}->local_name . ">";
+
+            #skip deleted elements from xml stream
+            unless ( $current_obj->is_delete_element ) {
+                if ( UNIVERSAL::isa( $self->{Handler}, 'XML::ExtOn' ) ) {
+                    my $cloned = $current_obj->__clone;
+                    unless ( $self->{__make_self_events} ) {
+                        $self->{Handler}->start_element($cloned);
+                    }
+                    else {
+                        $self->{Handler}->__start_element($cloned);
+
+                    }
+                }
+                else {
+                    my $res_data = $self->__exp_element_to_sax2($current_obj);
+                    $self->SUPER::start_element($res_data);
+                }
+            }
+        }
+        elsif ( $comm->{type} eq 'EV_END_ELEMENT' ) {
+            my $current_obj  = $comm->{data};
+            my $current_obj1 = pop @{ $self->_objects_stack() };
+
+      #warn "END_E: ".$current_obj->local_name;
+      # if skip
+      #check current element for skip_content
+      #            if ( my $current_element = $self->current_element ) {
+      #                my $skip_content = $current_element->is_skip_content;
+      #                if ( $skip_content > 1 ) {
+      #                    $current_element->is_skip_content( --$skip_content );
+      #                    return;
+      #                }
+      #            }
+
+            unless ( $current_obj->is_delete_element ) {
+                unless ( $self->{__make_self_events} ) {
+
+                    #convert to SAX2
+                    my $data = $current_obj->to_sax2;
+                    delete $data->{Attributes};
+                    $data->{NamespaceURI} = $current_obj->default_uri;
+                    $self->SUPER::end_element($data);
+                }
+                else {
+
+                    #call with object
+                    $self->{Handler}->__end_element($current_obj1);
+                }
+            }
+
+            my $changes    = $current_obj->ns->get_changes;
+            my $parent_map = $current_obj->ns->parent->get_map;
+            for ( keys %$changes ) {
+                $self->end_prefix_mapping(
+                    {
+                        Prefix       => $_,
+                        NamespaceURI => $changes->{$_},
+                    }
+                );
+                if ( exists( $parent_map->{$_} ) ) {
+                    $self->start_prefix_mapping(
+                        {
+                            Prefix       => $_,
+                            NamespaceURI => $parent_map->{$_},
+                        }
+                    );
+                }
+            }
+
+            #warn ref($self) . ": </" . $comm->{data}->local_name . ">";
 
-            # $self->end_element( $comm->{data} );
         }
     }
     else {
